from sensor.Sensor import Sensor
from utils.datatypes import *
from utils import i18n

import libdesklets as lib

import os
import time
import Queue


class SysInfoReader(Sensor):

    # the maximum number of fields
    SIZE = 5

    def __init__(self):

        global _; _ = i18n.Translator("sysinfo-sensor")

        self.__speed_in_max = 0
        self.__speed_out_max = 0


        self.__cpu_type = lib.cpu.get_model()
        self.__cpu_speed = lib.cpu.get_speed()
        self.__cpu_cache = lib.cpu.get_cache_size()

        self.__fonts = ""

        self.__key_color = ""
        self.__val_color = ""

        self.__current_size = 0
        self.__size_changed = 0

        self.__path  = os.path.join(os.getcwd(), "gfx")
        self.__queue = Queue.Queue(1)

        Sensor.__init__(self)

        self._set_config_type("update", TYPE_INT, 1000)
        self._set_config_type("fonts", TYPE_STRING, "Sans 10")
        self._set_config_type("key_color", TYPE_STRING, "white")
        self._set_config_type("val_color", TYPE_STRING, "yellow")
        self._set_config_type("memory", TYPE_STRING, "RAM")
        self._set_config_type("partition", TYPE_STRING, "/")
        self._set_config_type("network", TYPE_STRING, "eth0")
        self._set_config_type("show_cpu", TYPE_BOOL, 1)
        self._set_config_type("show_hdd", TYPE_BOOL, 1)
        self._set_config_type("show_ram", TYPE_BOOL, 1)
        self._set_config_type("show_net", TYPE_BOOL, 1)

        self._add_thread(self.__get_sys_info_thread)
        self._add_timer(100, self.__get_sys_info)



    def get_configurator(self):

        part = []
        for dev, mpoint in lib.disk.get_partitions():
            part.append([dev, mpoint])

        net = []
        for dev in lib.net.get_devices():
            net.append([dev, dev])

        mem = [['RAM', 'RAM'], ['Swap', 'Swap']]

        configurator = self._new_configurator()

        configurator.set_name(_("SysInfo"))
        configurator.add_title(_("Settings"))
        configurator.add_checkbox(_("Show CPU Information"), "show_cpu",
                                  _("If enabled, shows the CPU display"))
        configurator.add_checkbox(_("Show Harddisk Information"), "show_hdd",
                                  _("If enabled, shows the HDD display"))
        configurator.add_checkbox(_("Show RAM Information"), "show_ram",
                                  _("If enabled, shows the RAM display"))
        configurator.add_checkbox(_("Show Network Traffic"), "show_net",
                                  _("If enabled, shows the network traffic "
                                    "display"))
        configurator.add_title(_("WARNING:\nIf you set the update rate to a "
                                 "LOW value,\nit will result in a HIGH load!"))
        configurator.add_spin(_("Update every (ms):"), "update",
                              _("Updates the display(s) every x milliseconds"),
                              1000, 10000)
        configurator.add_font_selector(_("Font:"), "fonts", _("The font which "
                                       "is used to display the labels"))
        configurator.add_color_selector(_("Key color:"), "key_color",
                                        _("The color of the key"))
        configurator.add_color_selector(_("Value color:"), "val_color",
                                        _("The color of the value"))
        configurator.add_option(_("Partition:"), "partition",
                                "Specify partition type", part)
        configurator.add_option(_("Memory:"), "memory",
                                "Specify whether RAM or Swap should be "
                                "displayed", mem)
        configurator.add_option(_("Network:"), "network",
                                "Specify network device", net)

        return configurator



    def __human_readable(self, value):

        if value >= 1099511627776:
            return "%.2f TB" % (value / (1024.0**4))
        if value >= 1073741824:
            return "%.2f GB" % (value / (1024.0**3))
        if value >= 1048576:
            return "%.2f MB" % (value / (1024.0**2))
        if value >= 1024:
            return "%.2f kB" % (value / 1024.0)
        else:
            return "%i" % value



    def __do_cpu(self, signal, index):

        total, load = lib.cpu.get_load()
        signal.set("icon[%(index)d]"   % vars(), os.path.join(self.__path,
                                                              "cpu.png"))
        signal.set("key1[%(index)d]"   % vars(), _("Type:"))
        signal.set("value1[%(index)d]" % vars(), self.__cpu_type)
        signal.set("key2[%(index)d]"   % vars(), _("Clock:"))
        signal.set("value2[%(index)d]" % vars(), self.__cpu_speed)
        signal.set("key3[%(index)d]"   % vars(), _("Cache:"))
        signal.set("value3[%(index)d]" % vars(), self.__cpu_cache)
        signal.set("fill[%(index)d]"   % vars(), load)
        signal.set("show1[%(index)d]"  % vars(), 1)
        signal.set("show2[%(index)d]"  % vars(), 1)
        signal.set("show3[%(index)d]"  % vars(), 1)
        signal.set("showgauge[%(index)d]" % vars(), 1)
        signal.set("showplot[%(index)d]"  % vars(), 0)



    def __do_hdd(self, signal, index):

        part = self._get_config("partition")
        hdd_total, hdd_used = lib.disk.get_size(part)
        hdd_free = hdd_total - hdd_used
        hdd_perc = (hdd_used / hdd_total) * 100

        signal.set("icon[%(index)d]"      % vars(), os.path.join(self.__path,
                                                                 "hdd.png"))
        signal.set("key1[%(index)d]"      % vars(), _("Partition:"))
        signal.set("value1[%(index)d]"    % vars(), part)
        signal.set("key2[%(index)d]"      % vars(), _("Total:"))
        signal.set("value2[%(index)d]"    % vars(),
                   self.__human_readable(hdd_total))
        signal.set("key3[%(index)d]"      % vars(), _("Used:"))
        signal.set("value3[%(index)d]"    % vars(),
                   self.__human_readable(hdd_used))
        signal.set("key4[%(index)d]"      % vars(), _("Free:"))
        signal.set("value4[%(index)d]"    % vars(),
                   self.__human_readable(hdd_free))
        signal.set("fill[%(index)d]"      % vars(), "%.2f" % hdd_perc)
        signal.set("show1[%(index)d]"     % vars(), 1)
        signal.set("show2[%(index)d]"     % vars(), 1)
        signal.set("show3[%(index)d]"     % vars(), 1)
        signal.set("show4[%(index)d]"     % vars(), 1)
        signal.set("showgauge[%(index)d]" % vars(), 1)
        signal.set("showplot[%(index)d]"  % vars(), 0)



    def __do_ram(self, signal, index):

        mem = self._get_config("memory")
        if (mem == "RAM"):
            total, used = lib.memory.get_ram()
        else:
            total, used = lib.memory.get_swap()

        if (total != 0):
           free = total - used
           perc = (used / float(total)) * 100
        else:
            free = 0
            perc = 0

        signal.set("icon[%(index)d]"   % vars(), os.path.join(self.__path,
                                                              "memory.png"))
        signal.set("key1[%(index)d]"   % vars(), _("Total:"))
        signal.set("value1[%(index)d]" % vars(), self.__human_readable(1024 * total))
        signal.set("key2[%(index)d]"   % vars(), _("Used:"))
        signal.set("value2[%(index)d]" % vars(), self.__human_readable(1024 * used))
        signal.set("key3[%(index)d]"   % vars(), _("Free:"))
        signal.set("value3[%(index)d]" % vars(), self.__human_readable(1024 * free))
        signal.set("fill[%(index)d]"   % vars(), "%.2f" % perc)
        signal.set("show1[%(index)d]"  % vars(), 1)
        signal.set("show2[%(index)d]"  % vars(), 1)
        signal.set("show3[%(index)d]"  % vars(), 1)
        signal.set("showgauge[%(index)d]" % vars(), 1)
        signal.set("showplot[%(index)d]"  % vars(), 0)



    def __do_sys(self, signal, index):

        l1 = lib.sys.get_load_avg_1m()
        l5 = lib.sys.get_load_avg_5m()
        l15 = lib.sys.get_load_avg_15m()
        tasks, running = lib.sys.get_tasks()
        users = lib.sys.get_users()
        #d_start, h_start, m_start, s_start = lib.convert.secs_to_dhms(start)
        uptime = lib.sys.get_uptime()
        d, h, m, s = lib.convert.secs_to_dhms(uptime)
        idletime = lib.sys.get_idletime()
        idle_d, idle_h, idle_m, idle_s = lib.convert.secs_to_dhms(idletime)
        idle_p = (idletime / float(uptime)) * 100

        signal.set("icon[%(index)d]"      % vars(), os.path.join(self.__path,
                                                                 "SysInfo.png"))
        signal.set("key1[%(index)d]"      % vars(), _("Load:"))
        signal.set("value1[%(index)d]"    % vars(), str(l1) + " " + str(l5) + " " + str(l15))
        signal.set("key2[%(index)d]"      % vars(), _("Users:"))
        signal.set("value2[%(index)d]"    % vars(), users)
        signal.set("key3[%(index)d]"      % vars(), _("Uptime:"))
        signal.set("value3[%(index)d]"    % vars(), str(d) + "d " + \
                    str(h) + "h " + str(m) + "m ")
        signal.set("key4[%(index)d]"      % vars(), _("Idle time:"))
        signal.set("value4[%(index)d]"    % vars(), str(idle_d) + "d " + \
                    str(idle_h) + "h " + str(idle_m) + "m ")
        signal.set("fill[%(index)d]"   % vars(), "%.2f" %idle_p)
        signal.set("show1[%(index)d]"     % vars(), 1)
        signal.set("show2[%(index)d]"     % vars(), 1)
        signal.set("show3[%(index)d]"     % vars(), 1)
        signal.set("show4[%(index)d]"     % vars(), 1)
        signal.set("showgauge[%(index)d]" % vars(), 1)
        signal.set("showplot[%(index)d]"  % vars(), 0)



    def __do_net(self, signal, index):

        net = self._get_config("network")
        bytes_in, bytes_out = lib.net.get_bytes_in_out(net)
        pack_in, pack_out = lib.net.get_pack_in_out(net)
        speed_in, speed_out = lib.net.get_speed_in_out(net)
        ipaddr = lib.net.get_ipaddr(net)

        # compute maxima and percents
        #self.__speed_in_max = max(self.__speed_in_max, speed_in)
        #self.__speed_out_max = max(self.__speed_out_max, speed_out)

        if (self.__speed_in_max != 0):
            speed_in_percent = 100 * speed_in / self.__speed_in_max
        else:
            speed_in_percent = 0

        if (self.__speed_out_max != 0):
            speed_out_percent = 100 * speed_out / self.__speed_out_max
        else:
            speed_out_percent = 0

        signal.set("icon[%(index)d]"      % vars(),
                   os.path.join(self.__path, "network.png"))
        signal.set("key1[%(index)d]"      % vars(), _("Device:"))
        signal.set("value1[%(index)d]"    % vars(), net)
        signal.set("key2[%(index)d]"      % vars(), _("Bytes in:"))
        signal.set("value2[%(index)d]"    % vars(),
                   self.__human_readable(bytes_in))
        signal.set("key3[%(index)d]"      % vars(), _("Bytes out:"))
        signal.set("value3[%(index)d]"    % vars(),
                   self.__human_readable(bytes_out))
        signal.set("key4[%(index)d]"      % vars(), _("IP address:"))
        signal.set("value4[%(index)d]"    % vars(), ipaddr)
        signal.set("plot1[%(index)d]"     % vars(), speed_in)
        signal.set("plot2[%(index)d]"     % vars(), speed_out)
        signal.set("show1[%(index)d]"     % vars(), 1)
        signal.set("show2[%(index)d]"     % vars(), 1)
        signal.set("show3[%(index)d]"     % vars(), 1)
        signal.set("show4[%(index)d]"     % vars(), 1)
        signal.set("showgauge[%(index)d]" % vars(), 0)
        signal.set("showplot[%(index)d]"  % vars(), 1)



    def __update_config(self):

        update    = self._get_config("update")
        fonts     = self._get_config("fonts")
        key_color = self._get_config("key_color")
        val_color = self._get_config("val_color")

        v1 = (self.__fonts, self.__key_color, self.__val_color)
        v2 = (fonts, key_color, val_color)

        # only take action when some value has changed
        if (v1 != v2 or self.__size_changed):
            signal = self._new_output()

            if (self.__fonts != fonts or self.__size_changed):
                for i in xrange(self.SIZE):
                    signal.set("fonts[%(i)d]" %vars(), fonts)
            if (self.__key_color != key_color or self.__size_changed):
                for i in xrange(self.SIZE):
                    signal.set("key_color[%(i)d]" %vars(), key_color)
            if (self.__val_color != val_color or self.__size_changed):
                for i in xrange(self.SIZE):
                    signal.set("val_color[%(i)d]" %vars(), val_color)

            self._send_output(signal)

            self.__fonts        = fonts
            self.__key_color    = key_color
            self.__val_color    = val_color
            self.__size_changed = 0



    def __get_sys_info(self):

        self.__update_config()
        if (not self.__queue.empty()):
            signal = self.__queue.get()
            self._send_output(signal)

            size = signal.get("size")
            if (size != self.__current_size):
                self.__current_size = size
                self.__size_changed = 1

        return 1



    def __get_sys_info_thread(self):

        while(1):
            if (self._is_stopped()): break
            index     = 0
            update    = self._get_config("update")

            signal = self._new_output()

            self.__do_sys(signal, index)
            index += 1

            try:
                if (self._get_config("show_cpu")):
                    self.__do_cpu(signal, index)
                    index += 1
                if (self._get_config("show_ram")):
                    self.__do_ram(signal, index)
                    index += 1
                if (self._get_config("show_hdd")):
                    self.__do_hdd(signal, index)
                    index += 1
                if (self._get_config("show_net")):
                    self.__do_net(signal, index)
                    index += 1
            except:
                import traceback; traceback.print_exc()

            signal.set("size", index)

            self.__queue.put(signal)
            time.sleep(update / 1000)



def new_sensor(args): return apply(SysInfoReader, args)
