from sensor.Sensor import Sensor
from utils.datatypes import *
from utils import i18n


import commands
import re
import os
import time
import Queue
import urllib
import string


#
#  TempNOAA sensors
#      grabs temperature evolution of airports from the noaa website
#

class TempNOAASensor(Sensor):

    TEMPERATURE_SOURCE = "http://weather.noaa.gov/weather/current/%s.html"
    DEFAULT_MIN        = "0"
    DEFAULT_MAX        = "30"
    DEFAULT_CODE       = "LFBD"
    DEFAULT_UNIT       = "C"


    def __init__(self):
        self.__last_fetch = 0
        self.__queue = Queue.Queue(1)
        
        Sensor.__init__(self)

        self._set_config_type("code", TYPE_STRING, self.DEFAULT_CODE)
        self._set_config_type("min",  TYPE_STRING, self.DEFAULT_MIN)
        self._set_config_type("max",  TYPE_STRING, self.DEFAULT_MAX)
        self._set_config_type("unit", TYPE_STRING, self.DEFAULT_UNIT)
        
        self._add_thread(self.__get_temperature_thread)
        self._add_timer(0, self.__get_temperature)



    #
    # Returns the configuration GUI.
    #

    def get_configurator(self):

        configurator = self._new_configurator()
        configurator.set_name("Configuration")

        configurator.add_title("Source Settings")
        configurator.add_entry("Airport code :",
                               "code",
                               "The International airport code")
        

        configurator.add_title("Graph Settings")
        configurator.add_entry("Min Temp. :",
                               "min",
                               "Minimum temperature")
        configurator.add_entry("Max Temp. :",
                               "max",
                               "Maximum temperature")

        configurator.add_title("Display Settings")
        configurator.add_option("Unit :", "unit",
                                "Display Unit",[["Celcius", "C"], ["Farhenheit", "F"]])
        
        return configurator
    
    #
    # sends temperature data if available
    #
    
    def __get_temperature(self):

        # check the queue regularly
        if (not self.__queue.empty()):
            output = self.__queue.get()
            self._send_output(output)

        self._add_timer(500, self.__get_temperature)
        

    #
    # fetchs temperatures from noaa website.
    # returns a list of temperatures in centigrade unit


    def  __fetch_temperature (self,code):
        list = []
        f = urllib.urlopen(self.TEMPERATURE_SOURCE % code)
        lines = f.readlines()
        count=0;
        for line in lines:
            m = re.search ("(-?[0-9]+) \((-?[0-9]+)\)", line)
            if m is not None:
                if (count % 3)==0:
                    list.append((m.group(2), m.group(1)))
                count = count + 1
            m = re.search("No Data", line)
            if m is not None:
                list.append((None,None))

        if len(list)!=24:
            return None
        else:
            return list


    #
    # return the graph string
    #

    def __graph (self, list, min, max, unit):

        if (min == max or list == None):
            return ""
        
        graph = "color black,"

        # grid
        for i in range (int(min/10), int (max/10 + 1)):
            y = -1. + (2. * (int(i*10) - min)) / (max-min)
            if i==0:
                graph = graph + "color grey,"
            graph = graph + "line -1 %f 1 %f, "%(y, y)
            if i==0:
                graph = graph + "color black,"
        
        graph = graph + "color white,"
        for i in range(24):
            val = list[i]
            x = 1 - (2. * i / 24)
            if val is not None:
                y = -1. + (2. * (int(val) - min)) / (max-min)
                if i>0:
                    if y is not None and oldy is not None:
                        graph = graph + " line %f %f %f %f" % (oldx, oldy, x, y)
                        if i!=23:
                            graph = graph + ","
            else:
                y = None
                
            oldx = x
            oldy = y

        return graph

    #
    # Thread function for retrieving the weather data since this may take time
    # and must not block the application.
    #
    def __get_temperature_thread(self):
        
        min = 0
        max = 0
        code = ""
        unit = ""

        (oldmin, oldmax, oldcode, oldunit) = (min, max, code, unit)

        while (1):
            if (self._is_stopped()): break

            now = int(time.time())
            last = now - self.__last_fetch

            try:
                min = int(self._get_config("min"))
                max = int(self._get_config("max"))
            except:
                (min, max) = (oldmin, oldmax)

            code = self._get_config("code")
            unit = self._get_config("unit")


            fetch = 0
            draw = 0

            if last>(20*60) or (oldcode!=code):
                fetch = 1
                draw = 1

            if (oldmin, oldmax, oldunit) != (min, max, unit):
                draw = 1

            if (fetch==1):
                self.__list = self.__fetch_temperature(code)                
                self.__last_fetch = now

            if (draw==1):
                output = self._new_output()
                if self.__list is not None:
                    if (unit=="F"):
                        list = [ f for (c,f) in self.__list ]
                    else:
                        list = [ c for (c,f) in self.__list ]

                    output.set("value", " %s %s"%(str(list[0]), unit))
                    output.set("data", "clear, " +
                               self.__graph(list, min, max, unit))
            
                    self.__queue.put(output)                

            (oldmin, oldmax, oldcode, oldunit) = (min, max, code, unit)

            time.sleep(1)
        #end while

def new_sensor(args): return apply(TempNOAASensor, args)
